using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Web.Caching;
using System.Collections.Generic;
using PigeonCms;
using System.IO;
using System.Globalization;

public partial class Controls_Default : PigeonCms.BaseModuleControl
{
    const int DaysToShow = 14;
    const DayOfWeek FirstDayName = DayOfWeek.Monday;


    protected DateTime EventStart
    {
        get
        {
            CultureInfo culture;
            DateTimeStyles styles;
            culture = CultureInfo.CreateSpecificCulture("it-IT");
            styles = DateTimeStyles.None;
            DateTime res;
            DateTime.TryParse(TxtEventStart.Text, culture, styles, out res);
            return res.Date;
        }
        set
        {
            TxtEventStart.Text = value.ToShortDateString();
        }
    }

    protected DateTime EventEnd
    {
        get
        {
            CultureInfo culture;
            DateTimeStyles styles;
            culture = CultureInfo.CreateSpecificCulture("it-IT");
            styles = DateTimeStyles.None;
            DateTime res;
            DateTime.TryParse(TxtEventEnd.Text, culture, styles, out res);
            return res.Date;
        }
        set
        {
            TxtEventEnd.Text = value.ToShortDateString();
        }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        LblOk.Text = "";
        LblErr.Text = "";

        if (!Page.IsPostBack)
        {
            loadDropItems();
            loadDropStatus();
            Cal1.SelectedDate = DateTime.Today;
            setCalendarSelection();

            buildDayPilot();
            loadDayPilot();
        }
    }

    protected void BtnNew_Click(object sender, EventArgs e)
    {
        editRow(0);
    }

    protected void BtnSave_Click(object sender, EventArgs e)
    {
        LblErr.Text = "";
        LblOk.Text = "";
        try
        {
            var o1 = new Event();
            var mgr = new EventsManager<Event, EventsFilter>();
            if (CurrentId == 0)
            {
                form2obj(o1);
                o1 = mgr.Insert(o1, true);
            }
            else
            {
                o1 = mgr.GetByKey(CurrentId);
                form2obj(o1);
                mgr.Update(o1, true);
            }
            loadDayPilot();
            LblOk.Text = Utility.GetLabel("RECORD_SAVED_MSG");
            MultiView1.ActiveViewIndex = 0;
        }
        catch (ArgumentException ex1)
        {
            if (ex1.ParamName == "NotAvailable")
                LblErr.Text = "Periodo non disponibile";
            else
                LblErr.Text = Utility.GetLabel("RECORD_ERR_MSG") + "<br />" + ex1.ToString();
        }
        catch (Exception ex2)
        {
            LblErr.Text = Utility.GetLabel("RECORD_ERR_MSG") + "<br />" + ex2.ToString();
        }
        finally
        {
        }
    }

    protected void BtnCancel_Click(object sender, EventArgs e)
    {
        LblErr.Text = "";
        LblOk.Text = "";
        MultiView1.ActiveViewIndex = 0;
    }

    protected void BtnDelete_Click(object sender, EventArgs e)
    {
        LblErr.Text = "";
        LblOk.Text = "";
        try
        {
            var mgr = new EventsManager<Event, EventsFilter>();
            if (CurrentId > 0)
            {
                mgr.DeleteById(CurrentId);
            }
            loadDayPilot();
            LblOk.Text = Utility.GetLabel("RECORD_DELETED_MSG");
            MultiView1.ActiveViewIndex = 0;
        }
        catch (Exception ex2)
        {
            LblErr.Text = Utility.GetLabel("RECORD_ERR_MSG") + "<br />" + ex2.ToString();
        }
        finally
        {
        }
    }

    protected void MultiView1_ActiveViewChanged(object sender, EventArgs e)
    {
        if (MultiView1.ActiveViewIndex == 0)    //list view
        {
        }

        if (MultiView1.ActiveViewIndex == 1)    //edit view
        {
        }
    }

    #region DayPilot

    protected void DayPilot1_EventClick(object sender, DayPilot.Web.Ui.EventClickEventArgs e)
    {
        editRow(int.Parse(e.Value));
        //deleteRow(int.Parse(e.Value));
    }
    protected void DayPilot1_FreeTimeClick(object sender, DayPilot.Web.Ui.FreeClickEventArgs e)
    {
        editRow(0, e.Start, int.Parse(e.Resource));
    }
    protected void Cal1_SelectionChanged(object sender, EventArgs e)
    {
        setCalendarSelection();
        loadDayPilot();
    }

    private void setCalendarSelection()
    {
        DateTime firstDay = Utility.FirstDayOfWeek(Cal1.SelectedDate, FirstDayName);
        Cal1.VisibleDate = firstDay;
        for (int i = 0; i < DaysToShow; i++)
            Cal1.SelectedDates.Add(firstDay.AddDays(i));

        DayPilot1.StartDate = firstDay;
    }

    private void buildDayPilot()
    {
        var roomsMan = new RoomItemsManager(false);
        var roomsList = new List<RoomItem>();
        var filter = new RoomItemFilter();

        roomsList = roomsMan.GetByFilter(filter, "");
        foreach (var room in roomsList)
        {
            string desc = "<div class='DayPilotEventHeader'>" +
                "<b>" + room.Title + "</b><br />" +
                "posti letto: " + room.Beds.ToString() + "<br />" +
                "prezzo per notte: " + room.Price.ToString() + "";
            DayPilot1.Resources.Add(desc, room.Id.ToString());
        }

        DayPilot1.Days = DaysToShow;
        //DayPilot1.DurationBarColor = System.Drawing.Color.Red;
        DayPilot1.RowHeaderWidth = 180;
        DayPilot1.CellWidth = 50;
        DayPilot1.EventHeight = 50;
    }

    private void loadDayPilot()
    {
        var eventsList = new List<Event>();
        var filter = new EventsFilter();
        ArrayList al = new ArrayList(); //daypilot ds

        filter.EventStart = Utility.FirstDayOfWeek(Cal1.SelectedDate, FirstDayName);
        filter.EventEnd = filter.EventStart.AddDays(DaysToShow);
        eventsList = new EventsManager<Event, EventsFilter>().GetByFilter(filter, "");

        foreach (var eve in eventsList)
        {
            string cssClass = "DayPilotEventToConfirm";
            if (eve.Status == Event.EventStatusEnum.Confirmed)
                cssClass = "DayPilotEventConfirmed";
            eve.Description = "<div class='DayPilotEvent " + cssClass + "'>" +
                eve.Name + "<br />" + eve.Description +
                "</div>";
            al.Add(eve);
        }

        DayPilot1.DataSource = al;
        DayPilot1.DataBind();
    }

    #endregion


    #region private methods
    private void clearForm()
    {
        TxtName.Text = "";
        TxtDescription.Text = "";
        TxtEventStart.Text = "";
        TxtEventEnd.Text = "";
    }

    private void form2obj(Event obj)
    {
        int resourceId = 0;

        obj.Id = CurrentId;
        obj.Name = TxtName.Text;
        obj.Description = TxtDescription.Text;
        obj.Status = (Event.EventStatusEnum)Enum.Parse(typeof(Event.EventStatusEnum), DropStatus.SelectedValue);

        int.TryParse(DropItems.SelectedValue, out resourceId);
        obj.ResourceId = resourceId;
        obj.EventStart = this.EventStart;
        obj.EventEnd = this.EventEnd;
    }

    private void obj2form(Event obj)
    {
        TxtName.Text = obj.Name;
        TxtDescription.Text = obj.Description;
        Utility.SetDropByValue(DropStatus, ((int)obj.Status).ToString());
        Utility.SetDropByValue(DropItems, obj.ResourceId.ToString());
        this.EventStart = obj.EventStart;
        this.EventEnd = obj.EventEnd;
        //TxtEventStart.Text = obj.EventStart.ToShortDateString();
        //TxtEventEnd.Text = obj.EventEnd.ToShortDateString();
    }

    private void editRow(int recordId)
    {
        editRow(recordId, DateTime.MinValue, 0);
    }

    private void editRow(int recordId, DateTime eventStart, int resourceId)
    {
        var obj = new Event();
        LblOk.Text = "";
        LblErr.Text = "";

        clearForm();
        CurrentId = recordId;
        if (CurrentId == 0)
        {
            obj.ResourceId = resourceId;
            if (eventStart != DateTime.MinValue)
            {
                obj.EventStart = eventStart;
                obj.EventEnd = eventStart.AddDays(1);
            }
            else
            {
                obj.EventStart = DateTime.Now;
                obj.EventEnd = DateTime.Now.AddDays(1);
            }
            obj2form(obj);
        }
        else
        {
            obj = new EventsManager<Event, EventsFilter>().GetByKey(CurrentId);
            obj2form(obj);
        }
        MultiView1.ActiveViewIndex = 1;
    }

    private void deleteRow(int recordId)
    {
        LblOk.Text = "";
        LblErr.Text = "";

        try
        {
            new EventsManager<Event, EventsFilter>().DeleteById(recordId);
        }
        catch (Exception e)
        {
            LblErr.Text = e.Message;
        }
        loadDayPilot();
    }

    private void loadDropItems()
    {
        DropItems.Items.Clear();
        DropItems.Items.Add(new ListItem(Utility.GetLabel("LblSelectItem", "Select item"), ""));

        var filter = new RoomItemFilter();
        List<RoomItem> items = new RoomItemsManager(false).GetByFilter(filter, "");
        foreach (var item in items)
        {
            DropItems.Items.Add(new ListItem(item.Title, item.Id.ToString()));
        }
    }

    private void loadDropStatus()
    {
        DropStatus.Items.Clear();
        DropStatus.Items.Add(new ListItem("da confermare", "0"));
        DropStatus.Items.Add(new ListItem("confermato", "1"));

        //foreach (string item in Enum.GetNames(typeof(Event.EventStatusEnum)))
        //{
        //    int value = (int)Enum.Parse(typeof(Event.EventStatusEnum), item);
        //    ListItem listItem = new ListItem(item, value.ToString());
        //    DropStatus.Items.Add(listItem);
        //}
    }


    #endregion
}
